package it.unicam.ids.RESTaurant4.sala.service;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import it.unicam.ids.RESTaurant4.sala.entity.Quote;
import it.unicam.ids.RESTaurant4.sala.entity.Tavolo;
import it.unicam.ids.RESTaurant4.sala.entity.TavoloRepository;
import it.unicam.ids.RESTaurant4.servizio.service.GestoreComande;
import it.unicam.ids.RESTaurant4.util.DettagliComanda;
import it.unicam.ids.RESTaurant4.util.DettagliTavolo;
import it.unicam.ids.RESTaurant4.util.TipoTavoloAttivo;

@Service
//@ComponentScan("it.unicam.ids.entity")
//@DependsOn("TavoloRepository")
public class Sala {

	private static final Logger log = LoggerFactory.getLogger(Sala.class);

	private ArrayList<Tavolo> tavoliLiberi;
	private ArrayList<Tavolo> tavoliOccupati;
	private ArrayList<TavoloSala> tavoliSala;
	private ArrayList<TavoloAsporto> tavoliAsporto;

	@Autowired
	private GestoreComande gComande;

	@Autowired
	private TavoloRepository tavoloRepository;

	private Sala() {
		this.tavoliLiberi = new ArrayList<Tavolo>();
		this.tavoliOccupati = new ArrayList<Tavolo>();
		this.tavoliSala = new ArrayList<TavoloSala>();
		this.tavoliAsporto = new ArrayList<TavoloAsporto>();

//		setSala(tavoloRepository.findAll());
	}

	public void addTavolo(Tavolo tavolo) {
		tavoliLiberi.add(tavolo);
	}

	public List<DettagliTavolo> assegnazioneTavolo() {
		ArrayList<DettagliTavolo> statoSala = new ArrayList<DettagliTavolo>();
		Tavolo tavolo;
		DettagliTavolo datiTavolo;
		Iterator<Tavolo> tli = tavoliLiberi.iterator();
		while (tli.hasNext()) {
			tavolo = tli.next();
			datiTavolo = new DettagliTavolo(tavolo.getID(), "immagine predefinita", tavolo.getCapienza(), false, false);
			statoSala.add(datiTavolo);
		}
		TavoloSala tavoloSala;
		Iterator<TavoloSala> tls = tavoliSala.iterator();
		while (tls.hasNext()) {
			tavoloSala = tls.next();
			if (tavoloSala.isComposito()) {
				datiTavolo = new DettagliTavolo(tavoloSala.getID(), "immagine predefinita", tavoloSala.getCapienza(),
						true, true);
				Tavolo[] tavoliComponenti = tavoloSala.getTavoliComponenti();
				for (int i = 0; i < tavoliComponenti.length; i++) {
					datiTavolo.addTavoloComponente(tavoliComponenti[i].getID());
				}
			} else {
				datiTavolo = new DettagliTavolo(tavoloSala.getID(), "immagine predefinita", tavoloSala.getCapienza(),
						true, false);

			}
			statoSala.add(datiTavolo);
		}
		return statoSala;
	}

	/**
	 * 
	 * @param numCoperti
	 */
	public ArrayList<Long> getTavolo(int numCoperti) {
		return this.suggerisciTavoli(numCoperti);
	}

	/**
	 * 
	 * @param idTavolo
	 */
	public void riservaTavolo(Long idTavolo, int numCoperti) {
		Tavolo tavolo = occupaTavolo(idTavolo);
		if (tavolo != null) {
			String idComanda = gComande.newComandaID(numCoperti);
			TavoloSala tavoloSala = new TavoloSala(tavolo, idComanda);
			tavoliSala.add(tavoloSala);
			log.info("[INFO] Il tavolo " + tavoloSala.getID() + " è stato assegnato");
		}
	}

	public void riservaTavolo(Long[] idTavolo, int numCoperti) {
		Tavolo[] tavoli = new Tavolo[idTavolo.length];
		Tavolo tavolo;
		int trovati = 0;
		for (int i = 0; i < idTavolo.length; i++) {
			tavolo = occupaTavolo(idTavolo[i]);
			if (tavolo != null) {
				tavoli[i] = tavolo;
				trovati++;
			} else {
				break;
			}
		}
		if (trovati < idTavolo.length) {
			System.out.println(
					"[ERROR]: Attenzione alcuni o tutti i codici inseriti non corrispondono a tavoli liberi. il tavolo composito non è stato creato ");
			for (int i = 0; i < trovati; i++) {
				liberaTavolo(tavoli[i]);
			}
			return;
		}
		String idComanda = gComande.newComandaID(numCoperti);
		Long newID = this.generaStampaID();
		TavoloSala tavoloSala = new TavoloSala(tavoli, idComanda);
		tavoloSala.setID(newID);
		tavoliSala.add(tavoloSala);
		System.out.println("[INFO] Generato nuovo ID per tavolo composito: " + newID);
		if (trovati < idTavolo.length)
			System.out.println("[ERROR]: Attenzione alcuni codici inseriti non corrispondevano a tavoli esistenti");
	}

	public Tavolo occupaTavolo(Long idTavolo) {

		RestTemplate restTemplate = new RestTemplate();
		Quote quote = restTemplate.getForObject("https://gturnquist-quoters.cfapps.io/api/random", Quote.class);
		log.info(quote.toString());

//		ArrayList<DettagliTavolo> adt = new ArrayList<DettagliTavolo>();
//		adt = restTemplate.getForObject("http://localhost:8080/assegnaTavolo/statoSala", ArrayList.class);
//		log.info(adt.toString());

		Iterator<Tavolo> tli = tavoliLiberi.iterator();
		Tavolo tavolo = null;
		boolean trovato = false;
		while (!trovato && tli.hasNext()) {
			tavolo = tli.next();
			if (tavolo.getID().equals(idTavolo)) {
				tavoliLiberi.remove(tavolo);
				tavoliOccupati.add(tavolo);
				trovato = true;
			}
		}
		if (trovato) {
			return tavolo;
		} else {
			System.out.println("[ERROR]: Il tavolo con ID " + idTavolo + " non risulta essere tra i liberi!");
			return null;
		}

	}

	public void liberaTavolo(Tavolo tavolo) {
		boolean rimosso = tavoliOccupati.remove(tavolo);
		if (!rimosso) {
			System.out.println("[INFO]: Attenzione il tavolo da liberare non risulta tra gli occupati");
			return;
		}
		tavoliLiberi.add(tavolo);
	}

	/**
	 * 
	 * @param numCoperti
	 */
	public ArrayList<Long> suggerisciTavoli(int numCoperti) {
		ArrayList<Long> selezioneTavoli = new ArrayList<Long>();
		Iterator<Tavolo> tli = tavoliLiberi.iterator();
		Tavolo tavolo;
		while (tli.hasNext()) {
			tavolo = tli.next();
			if (tavolo.getCapienza() >= numCoperti) {
				selezioneTavoli.add(tavolo.getID());
			}
		}
		return selezioneTavoli;
	}

	public Long generaStampaID() {
		Random random = new Random();
		Integer ID = random.nextInt(10000);
		return Integer.toUnsignedLong(ID);
	}

	/**
	 * 
	 * @param idTavolo
	 */
	public DettagliComanda compilaComandaTavolo(String idTavolo) {
		TavoloSala tavoloSala = this.getTavoloSala(idTavolo);
		if (tavoloSala == null)
			return null;
		String idComanda = tavoloSala.getComanda();
		DettagliComanda dettagli = gComande.getDettagliComanda(idComanda);
		return dettagli;
	}

	public TavoloSala getTavoloSala(String idTavolo) {
		Iterator<TavoloSala> tsi = tavoliSala.iterator();
		TavoloSala tavoloSala = null;
		boolean trovato = false;
		while (tsi.hasNext()) {
			tavoloSala = tsi.next();
			if (tavoloSala.getID().equals(idTavolo))
				trovato = true;
		}
		if (trovato) {
			return tavoloSala;
		} else {
			System.out.println("[ERROR]: Il tavolo con ID " + idTavolo + " non esiste!");
			return null;
		}
	}

	public void setSala(Iterable<Tavolo> tavoli) {
		Iterator<Tavolo> ti = tavoli.iterator();
		Tavolo tavolo;
		while (ti.hasNext()) {
			tavolo = ti.next();
			tavoliLiberi.add(tavolo);
			log.info("Added tavolo: " + tavolo);
		}
	}

	public void liberaTavolo(String idTavolo) {
		
		Iterator<TavoloSala> iTavoliSala = tavoliSala.iterator();
		TavoloSala tavoloSala = null;
		boolean esiste = false;
		while (iTavoliSala.hasNext()) {
			tavoloSala = iTavoliSala.next();
			if (tavoloSala.getID().equals(idTavolo))
				esiste = true;
		}
		if (esiste) {
		    if(!tavoloSala.isComposito()) {
		    System.out.println("[INFO]: Il tavolo con ID " + idTavolo + " non ci sono tavoli associati ");
			Iterator<Tavolo> tavoliOcc = tavoliOccupati.iterator();
			Tavolo tavolo = null;
			boolean trovato = false;
			while (!trovato && tavoliOcc.hasNext()) {
				tavolo = tavoliOcc.next();
				if (tavolo.getID().equals(idTavolo)) {
					tavoloSala.setTipo(TipoTavoloAttivo.inpagamento);
					tavoliOccupati.remove(tavolo);
					tavoliLiberi.add(tavolo);
					trovato = true;
				}
			}
		    
			if (!trovato) {
					System.out.println("[ERRORE]: Il tavolo con ID " + idTavolo + " non e' occupato");
			}
		    } else
		    {
				System.out.println("[INFO]: Il tavolo con Codice " + idTavolo
						+ " e\' composito, attivita' di rimozione tavoli compisti in corso ...");
		    	Iterator<Tavolo> tavoliOcc = tavoliOccupati.iterator();
				Tavolo tavolo = null;
				boolean trovato = false;
				while (!trovato && tavoliOcc.hasNext()) {
					tavolo = tavoliOcc.next();
					if (tavolo.getID().equals(idTavolo)) {
						tavoloSala.setTipo(TipoTavoloAttivo.inpagamento);
						Tavolo[] tavoliComp;
						tavoliComp = tavoloSala.getTavoliComponenti();
						for(int i=0; i<tavoliComp.length; i++) {
						liberaTavolo(tavoliComp[i]);
						trovato = true;
						}
					}
				}
			    
				if (!trovato) {
					System.out.println("[ERROR]: Il tavolo con ID " + idTavolo + " non e' tra gli occupati!");
				}
		    }
			
		
		} else {
			System.out.println("[ERROR]: Il tavolo con ID " + idTavolo + " non esiste!");
		}
	}

}
